# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, Iterable, Optional, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .value import (
    ValueResource,
    AsyncValueResource,
    ValueResourceWithRawResponse,
    AsyncValueResourceWithRawResponse,
    ValueResourceWithStreamingResponse,
    AsyncValueResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.accounts import token_list_params, token_create_params, token_update_params
from .permission_groups import (
    PermissionGroupsResource,
    AsyncPermissionGroupsResource,
    PermissionGroupsResourceWithRawResponse,
    AsyncPermissionGroupsResourceWithRawResponse,
    PermissionGroupsResourceWithStreamingResponse,
    AsyncPermissionGroupsResourceWithStreamingResponse,
)
from ....types.shared.token import Token
from ....types.shared_params.token_policy import TokenPolicy
from ....types.accounts.token_create_response import TokenCreateResponse
from ....types.accounts.token_delete_response import TokenDeleteResponse
from ....types.accounts.token_verify_response import TokenVerifyResponse

__all__ = ["TokensResource", "AsyncTokensResource"]


class TokensResource(SyncAPIResource):
    @cached_property
    def permission_groups(self) -> PermissionGroupsResource:
        return PermissionGroupsResource(self._client)

    @cached_property
    def value(self) -> ValueResource:
        return ValueResource(self._client)

    @cached_property
    def with_raw_response(self) -> TokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TokensResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        policies: Iterable[TokenPolicy],
        condition: token_create_params.Condition | Omit = omit,
        expires_on: Union[str, datetime] | Omit = omit,
        not_before: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenCreateResponse]:
        """
        Create a new Account Owned API token.

        Args:
          account_id: Account identifier tag.

          name: Token name.

          policies: List of access policies assigned to the token.

          expires_on: The expiration time on or after which the JWT MUST NOT be accepted for
              processing.

          not_before: The time before which the token MUST NOT be accepted for processing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/tokens", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "policies": policies,
                    "condition": condition,
                    "expires_on": expires_on,
                    "not_before": not_before,
                },
                token_create_params.TokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenCreateResponse]], ResultWrapper[TokenCreateResponse]),
        )

    def update(
        self,
        token_id: str,
        *,
        account_id: str,
        name: str,
        policies: Iterable[TokenPolicy],
        condition: token_update_params.Condition | Omit = omit,
        expires_on: Union[str, datetime] | Omit = omit,
        not_before: Union[str, datetime] | Omit = omit,
        status: Literal["active", "disabled", "expired"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Token]:
        """
        Update an existing token.

        Args:
          account_id: Account identifier tag.

          token_id: Token identifier tag.

          name: Token name.

          policies: List of access policies assigned to the token.

          expires_on: The expiration time on or after which the JWT MUST NOT be accepted for
              processing.

          not_before: The time before which the token MUST NOT be accepted for processing.

          status: Status of the token.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not token_id:
            raise ValueError(f"Expected a non-empty value for `token_id` but received {token_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/tokens/{token_id}", account_id=account_id, token_id=token_id),
            body=maybe_transform(
                {
                    "name": name,
                    "policies": policies,
                    "condition": condition,
                    "expires_on": expires_on,
                    "not_before": not_before,
                    "status": status,
                },
                token_update_params.TokenUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Token]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Token]], ResultWrapper[Token]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Token]:
        """
        List all Account Owned API tokens created for this account.

        Args:
          account_id: Account identifier tag.

          direction: Direction to order results.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/tokens", account_id=account_id),
            page=SyncV4PagePaginationArray[Token],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "page": page,
                        "per_page": per_page,
                    },
                    token_list_params.TokenListParams,
                ),
            ),
            model=Token,
        )

    def delete(
        self,
        token_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenDeleteResponse]:
        """
        Destroy an Account Owned API token.

        Args:
          account_id: Account identifier tag.

          token_id: Token identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not token_id:
            raise ValueError(f"Expected a non-empty value for `token_id` but received {token_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/tokens/{token_id}", account_id=account_id, token_id=token_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenDeleteResponse]], ResultWrapper[TokenDeleteResponse]),
        )

    def get(
        self,
        token_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Token]:
        """
        Get information about a specific Account Owned API token.

        Args:
          account_id: Account identifier tag.

          token_id: Token identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not token_id:
            raise ValueError(f"Expected a non-empty value for `token_id` but received {token_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/tokens/{token_id}", account_id=account_id, token_id=token_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Token]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Token]], ResultWrapper[Token]),
        )

    def verify(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenVerifyResponse]:
        """
        Test whether a token works.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/tokens/verify", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenVerifyResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenVerifyResponse]], ResultWrapper[TokenVerifyResponse]),
        )


class AsyncTokensResource(AsyncAPIResource):
    @cached_property
    def permission_groups(self) -> AsyncPermissionGroupsResource:
        return AsyncPermissionGroupsResource(self._client)

    @cached_property
    def value(self) -> AsyncValueResource:
        return AsyncValueResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTokensResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        policies: Iterable[TokenPolicy],
        condition: token_create_params.Condition | Omit = omit,
        expires_on: Union[str, datetime] | Omit = omit,
        not_before: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenCreateResponse]:
        """
        Create a new Account Owned API token.

        Args:
          account_id: Account identifier tag.

          name: Token name.

          policies: List of access policies assigned to the token.

          expires_on: The expiration time on or after which the JWT MUST NOT be accepted for
              processing.

          not_before: The time before which the token MUST NOT be accepted for processing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/tokens", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "policies": policies,
                    "condition": condition,
                    "expires_on": expires_on,
                    "not_before": not_before,
                },
                token_create_params.TokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenCreateResponse]], ResultWrapper[TokenCreateResponse]),
        )

    async def update(
        self,
        token_id: str,
        *,
        account_id: str,
        name: str,
        policies: Iterable[TokenPolicy],
        condition: token_update_params.Condition | Omit = omit,
        expires_on: Union[str, datetime] | Omit = omit,
        not_before: Union[str, datetime] | Omit = omit,
        status: Literal["active", "disabled", "expired"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Token]:
        """
        Update an existing token.

        Args:
          account_id: Account identifier tag.

          token_id: Token identifier tag.

          name: Token name.

          policies: List of access policies assigned to the token.

          expires_on: The expiration time on or after which the JWT MUST NOT be accepted for
              processing.

          not_before: The time before which the token MUST NOT be accepted for processing.

          status: Status of the token.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not token_id:
            raise ValueError(f"Expected a non-empty value for `token_id` but received {token_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/tokens/{token_id}", account_id=account_id, token_id=token_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "policies": policies,
                    "condition": condition,
                    "expires_on": expires_on,
                    "not_before": not_before,
                    "status": status,
                },
                token_update_params.TokenUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Token]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Token]], ResultWrapper[Token]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Token, AsyncV4PagePaginationArray[Token]]:
        """
        List all Account Owned API tokens created for this account.

        Args:
          account_id: Account identifier tag.

          direction: Direction to order results.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/tokens", account_id=account_id),
            page=AsyncV4PagePaginationArray[Token],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "page": page,
                        "per_page": per_page,
                    },
                    token_list_params.TokenListParams,
                ),
            ),
            model=Token,
        )

    async def delete(
        self,
        token_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenDeleteResponse]:
        """
        Destroy an Account Owned API token.

        Args:
          account_id: Account identifier tag.

          token_id: Token identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not token_id:
            raise ValueError(f"Expected a non-empty value for `token_id` but received {token_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/tokens/{token_id}", account_id=account_id, token_id=token_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenDeleteResponse]], ResultWrapper[TokenDeleteResponse]),
        )

    async def get(
        self,
        token_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Token]:
        """
        Get information about a specific Account Owned API token.

        Args:
          account_id: Account identifier tag.

          token_id: Token identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not token_id:
            raise ValueError(f"Expected a non-empty value for `token_id` but received {token_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/tokens/{token_id}", account_id=account_id, token_id=token_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Token]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Token]], ResultWrapper[Token]),
        )

    async def verify(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TokenVerifyResponse]:
        """
        Test whether a token works.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/tokens/verify", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenVerifyResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TokenVerifyResponse]], ResultWrapper[TokenVerifyResponse]),
        )


class TokensResourceWithRawResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.create = to_raw_response_wrapper(
            tokens.create,
        )
        self.update = to_raw_response_wrapper(
            tokens.update,
        )
        self.list = to_raw_response_wrapper(
            tokens.list,
        )
        self.delete = to_raw_response_wrapper(
            tokens.delete,
        )
        self.get = to_raw_response_wrapper(
            tokens.get,
        )
        self.verify = to_raw_response_wrapper(
            tokens.verify,
        )

    @cached_property
    def permission_groups(self) -> PermissionGroupsResourceWithRawResponse:
        return PermissionGroupsResourceWithRawResponse(self._tokens.permission_groups)

    @cached_property
    def value(self) -> ValueResourceWithRawResponse:
        return ValueResourceWithRawResponse(self._tokens.value)


class AsyncTokensResourceWithRawResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.create = async_to_raw_response_wrapper(
            tokens.create,
        )
        self.update = async_to_raw_response_wrapper(
            tokens.update,
        )
        self.list = async_to_raw_response_wrapper(
            tokens.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tokens.delete,
        )
        self.get = async_to_raw_response_wrapper(
            tokens.get,
        )
        self.verify = async_to_raw_response_wrapper(
            tokens.verify,
        )

    @cached_property
    def permission_groups(self) -> AsyncPermissionGroupsResourceWithRawResponse:
        return AsyncPermissionGroupsResourceWithRawResponse(self._tokens.permission_groups)

    @cached_property
    def value(self) -> AsyncValueResourceWithRawResponse:
        return AsyncValueResourceWithRawResponse(self._tokens.value)


class TokensResourceWithStreamingResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.create = to_streamed_response_wrapper(
            tokens.create,
        )
        self.update = to_streamed_response_wrapper(
            tokens.update,
        )
        self.list = to_streamed_response_wrapper(
            tokens.list,
        )
        self.delete = to_streamed_response_wrapper(
            tokens.delete,
        )
        self.get = to_streamed_response_wrapper(
            tokens.get,
        )
        self.verify = to_streamed_response_wrapper(
            tokens.verify,
        )

    @cached_property
    def permission_groups(self) -> PermissionGroupsResourceWithStreamingResponse:
        return PermissionGroupsResourceWithStreamingResponse(self._tokens.permission_groups)

    @cached_property
    def value(self) -> ValueResourceWithStreamingResponse:
        return ValueResourceWithStreamingResponse(self._tokens.value)


class AsyncTokensResourceWithStreamingResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.create = async_to_streamed_response_wrapper(
            tokens.create,
        )
        self.update = async_to_streamed_response_wrapper(
            tokens.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tokens.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tokens.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            tokens.get,
        )
        self.verify = async_to_streamed_response_wrapper(
            tokens.verify,
        )

    @cached_property
    def permission_groups(self) -> AsyncPermissionGroupsResourceWithStreamingResponse:
        return AsyncPermissionGroupsResourceWithStreamingResponse(self._tokens.permission_groups)

    @cached_property
    def value(self) -> AsyncValueResourceWithStreamingResponse:
        return AsyncValueResourceWithStreamingResponse(self._tokens.value)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AnnotationListParams"]


class AnnotationListParams(TypedDict, total=False):
    asn: int
    """Filters results by Autonomous System.

    Specify a single Autonomous System Number (ASN) as integer.
    """

    data_source: Annotated[
        Literal[
            "ALL",
            "AI_BOTS",
            "AI_GATEWAY",
            "BGP",
            "BOTS",
            "CONNECTION_ANOMALY",
            "CT",
            "DNS",
            "DNS_MAGNITUDE",
            "DNS_AS112",
            "DOS",
            "EMAIL_ROUTING",
            "EMAIL_SECURITY",
            "FW",
            "FW_PG",
            "HTTP",
            "HTTP_CONTROL",
            "HTTP_CRAWLER_REFERER",
            "HTTP_ORIGINS",
            "IQI",
            "LEAKED_CREDENTIALS",
            "NET",
            "ROBOTS_TXT",
            "SPEED",
            "WORKERS_AI",
        ],
        PropertyInfo(alias="dataSource"),
    ]
    """Filters results by data source."""

    date_end: Annotated[Union[str, datetime], PropertyInfo(alias="dateEnd", format="iso8601")]
    """End of the date range (inclusive)."""

    date_range: Annotated[str, PropertyInfo(alias="dateRange")]
    """Filters results by date range."""

    date_start: Annotated[Union[str, datetime], PropertyInfo(alias="dateStart", format="iso8601")]
    """Start of the date range (inclusive)."""

    event_type: Annotated[
        Literal["EVENT", "GENERAL", "OUTAGE", "PARTIAL_PROJECTION", "PIPELINE", "TRAFFIC_ANOMALY"],
        PropertyInfo(alias="eventType"),
    ]
    """Filters results by event type."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    limit: int
    """Limits the number of objects returned in the response."""

    location: str
    """Filters results by location. Specify an alpha-2 location code."""

    offset: int
    """Skips the specified number of objects before fetching the results."""

    origin: str
    """Filters results by origin."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from ......_types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......_base_client import make_request_options
from ......types.zero_trust.access.applications import policy_test_create_params
from ......types.zero_trust.access.applications.policy_test_get_response import PolicyTestGetResponse
from ......types.zero_trust.access.applications.policy_test_create_response import PolicyTestCreateResponse

__all__ = ["PolicyTestsResource", "AsyncPolicyTestsResource"]


class PolicyTestsResource(SyncAPIResource):
    @cached_property
    def users(self) -> UsersResource:
        return UsersResource(self._client)

    @cached_property
    def with_raw_response(self) -> PolicyTestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PolicyTestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PolicyTestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PolicyTestsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        policies: SequenceNotStr[policy_test_create_params.Policy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PolicyTestCreateResponse]:
        """
        Starts an Access policy test.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/access/policy-tests", account_id=account_id),
            body=maybe_transform({"policies": policies}, policy_test_create_params.PolicyTestCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PolicyTestCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PolicyTestCreateResponse]], ResultWrapper[PolicyTestCreateResponse]),
        )

    def get(
        self,
        policy_test_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PolicyTestGetResponse]:
        """
        Fetches the current status of a given Access policy test.

        Args:
          account_id: Identifier.

          policy_test_id: The UUID of the policy test.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_test_id:
            raise ValueError(f"Expected a non-empty value for `policy_test_id` but received {policy_test_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/access/policy-tests/{policy_test_id}",
                account_id=account_id,
                policy_test_id=policy_test_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PolicyTestGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PolicyTestGetResponse]], ResultWrapper[PolicyTestGetResponse]),
        )


class AsyncPolicyTestsResource(AsyncAPIResource):
    @cached_property
    def users(self) -> AsyncUsersResource:
        return AsyncUsersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPolicyTestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPolicyTestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPolicyTestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPolicyTestsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        policies: SequenceNotStr[policy_test_create_params.Policy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PolicyTestCreateResponse]:
        """
        Starts an Access policy test.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/access/policy-tests", account_id=account_id),
            body=await async_maybe_transform({"policies": policies}, policy_test_create_params.PolicyTestCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PolicyTestCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PolicyTestCreateResponse]], ResultWrapper[PolicyTestCreateResponse]),
        )

    async def get(
        self,
        policy_test_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PolicyTestGetResponse]:
        """
        Fetches the current status of a given Access policy test.

        Args:
          account_id: Identifier.

          policy_test_id: The UUID of the policy test.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_test_id:
            raise ValueError(f"Expected a non-empty value for `policy_test_id` but received {policy_test_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/access/policy-tests/{policy_test_id}",
                account_id=account_id,
                policy_test_id=policy_test_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PolicyTestGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PolicyTestGetResponse]], ResultWrapper[PolicyTestGetResponse]),
        )


class PolicyTestsResourceWithRawResponse:
    def __init__(self, policy_tests: PolicyTestsResource) -> None:
        self._policy_tests = policy_tests

        self.create = to_raw_response_wrapper(
            policy_tests.create,
        )
        self.get = to_raw_response_wrapper(
            policy_tests.get,
        )

    @cached_property
    def users(self) -> UsersResourceWithRawResponse:
        return UsersResourceWithRawResponse(self._policy_tests.users)


class AsyncPolicyTestsResourceWithRawResponse:
    def __init__(self, policy_tests: AsyncPolicyTestsResource) -> None:
        self._policy_tests = policy_tests

        self.create = async_to_raw_response_wrapper(
            policy_tests.create,
        )
        self.get = async_to_raw_response_wrapper(
            policy_tests.get,
        )

    @cached_property
    def users(self) -> AsyncUsersResourceWithRawResponse:
        return AsyncUsersResourceWithRawResponse(self._policy_tests.users)


class PolicyTestsResourceWithStreamingResponse:
    def __init__(self, policy_tests: PolicyTestsResource) -> None:
        self._policy_tests = policy_tests

        self.create = to_streamed_response_wrapper(
            policy_tests.create,
        )
        self.get = to_streamed_response_wrapper(
            policy_tests.get,
        )

    @cached_property
    def users(self) -> UsersResourceWithStreamingResponse:
        return UsersResourceWithStreamingResponse(self._policy_tests.users)


class AsyncPolicyTestsResourceWithStreamingResponse:
    def __init__(self, policy_tests: AsyncPolicyTestsResource) -> None:
        self._policy_tests = policy_tests

        self.create = async_to_streamed_response_wrapper(
            policy_tests.create,
        )
        self.get = async_to_streamed_response_wrapper(
            policy_tests.get,
        )

    @cached_property
    def users(self) -> AsyncUsersResourceWithStreamingResponse:
        return AsyncUsersResourceWithStreamingResponse(self._policy_tests.users)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SSOCreateParams"]


class SSOCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    email_domain: Required[str]
    """Email domain of the new SSO connector"""

    begin_verification: bool
    """Begin the verification process after creation"""

    use_fedramp_language: bool
    """Controls the display of FedRAMP language to the user during SSO login"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["Settings"]


class Settings(BaseModel):
    validation_default_mitigation_action: Optional[Literal["none", "log", "block"]] = None
    """
    The default mitigation action used when there is no mitigation action defined on
    the operation

    Mitigation actions are as follows:

    - `log` - log request when request does not conform to schema
    - `block` - deny access to the site when request does not conform to schema

    A special value of of `none` will skip running schema validation entirely for
    the request when there is no mitigation action defined on the operation
    """

    validation_override_mitigation_action: Optional[Literal["none"]] = None
    """When set, this overrides both zone level and operation level mitigation actions.

    - `none` will skip running schema validation entirely for the request
    - `null` indicates that no override is in place
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ....._models import BaseModel

__all__ = ["QuotaGetResponse"]


class QuotaGetResponse(BaseModel):
    quota: float
    """The remaining number of commands that can be initiated for an account"""

    quota_usage: float
    """The number of commands that have been initiated for an account"""

    reset_time: datetime
    """The time when the quota resets"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["GatewayFilter"]

GatewayFilter: TypeAlias = Literal["http", "dns", "l4", "egress", "dns_resolver"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .workflow import Workflow as Workflow
from .neo_event import NeoEvent as NeoEvent
from .configuration import Configuration as Configuration
from .neo_event_param import NeoEventParam as NeoEventParam
from .history_list_params import HistoryListParams as HistoryListParams
from .zaraz_update_params import ZarazUpdateParams as ZarazUpdateParams
from .config_update_params import ConfigUpdateParams as ConfigUpdateParams
from .history_list_response import HistoryListResponse as HistoryListResponse
from .history_update_params import HistoryUpdateParams as HistoryUpdateParams
from .publish_create_params import PublishCreateParams as PublishCreateParams
from .button_text_translation import ButtonTextTranslation as ButtonTextTranslation
from .publish_create_response import PublishCreateResponse as PublishCreateResponse
from .button_text_translation_param import ButtonTextTranslationParam as ButtonTextTranslationParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from ...._models import BaseModel

__all__ = [
    "QueryGetResponse",
    "QueryGetResponseItem",
    "QueryGetResponseItemParameters",
    "QueryGetResponseItemParametersStringMatch",
]


class QueryGetResponseItemParametersStringMatch(BaseModel):
    max_edit_distance: float

    pattern: str


class QueryGetResponseItemParameters(BaseModel):
    string_matches: List[QueryGetResponseItemParametersStringMatch]

    max_time: Optional[str] = None

    min_time: Optional[str] = None


class QueryGetResponseItem(BaseModel):
    created: str

    parameters: Optional[QueryGetResponseItemParameters] = None

    query_id: int

    query_tag: str

    scan: bool

    updated: str


QueryGetResponse: TypeAlias = List[QueryGetResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tokens import (
    TokensResource,
    AsyncTokensResource,
    TokensResourceWithRawResponse,
    AsyncTokensResourceWithRawResponse,
    TokensResourceWithStreamingResponse,
    AsyncTokensResourceWithStreamingResponse,
)
from .aisearch import (
    AISearchResource,
    AsyncAISearchResource,
    AISearchResourceWithRawResponse,
    AsyncAISearchResourceWithRawResponse,
    AISearchResourceWithStreamingResponse,
    AsyncAISearchResourceWithStreamingResponse,
)
from .instances import (
    InstancesResource,
    AsyncInstancesResource,
    InstancesResourceWithRawResponse,
    AsyncInstancesResourceWithRawResponse,
    InstancesResourceWithStreamingResponse,
    AsyncInstancesResourceWithStreamingResponse,
)
from .namespaces import (
    NamespacesResource,
    AsyncNamespacesResource,
    NamespacesResourceWithRawResponse,
    AsyncNamespacesResourceWithRawResponse,
    NamespacesResourceWithStreamingResponse,
    AsyncNamespacesResourceWithStreamingResponse,
)

__all__ = [
    "NamespacesResource",
    "AsyncNamespacesResource",
    "NamespacesResourceWithRawResponse",
    "AsyncNamespacesResourceWithRawResponse",
    "NamespacesResourceWithStreamingResponse",
    "AsyncNamespacesResourceWithStreamingResponse",
    "InstancesResource",
    "AsyncInstancesResource",
    "InstancesResourceWithRawResponse",
    "AsyncInstancesResourceWithRawResponse",
    "InstancesResourceWithStreamingResponse",
    "AsyncInstancesResourceWithStreamingResponse",
    "TokensResource",
    "AsyncTokensResource",
    "TokensResourceWithRawResponse",
    "AsyncTokensResourceWithRawResponse",
    "TokensResourceWithStreamingResponse",
    "AsyncTokensResourceWithStreamingResponse",
    "AISearchResource",
    "AsyncAISearchResource",
    "AISearchResourceWithRawResponse",
    "AsyncAISearchResourceWithRawResponse",
    "AISearchResourceWithStreamingResponse",
    "AsyncAISearchResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tokens import (
    TokensResource,
    AsyncTokensResource,
    TokensResourceWithRawResponse,
    AsyncTokensResourceWithRawResponse,
    TokensResourceWithStreamingResponse,
    AsyncTokensResourceWithStreamingResponse,
)
from .vpc_flows import (
    VPCFlowsResource,
    AsyncVPCFlowsResource,
    VPCFlowsResourceWithRawResponse,
    AsyncVPCFlowsResourceWithRawResponse,
    VPCFlowsResourceWithStreamingResponse,
    AsyncVPCFlowsResourceWithStreamingResponse,
)

__all__ = [
    "TokensResource",
    "AsyncTokensResource",
    "TokensResourceWithRawResponse",
    "AsyncTokensResourceWithRawResponse",
    "TokensResourceWithStreamingResponse",
    "AsyncTokensResourceWithStreamingResponse",
    "VPCFlowsResource",
    "AsyncVPCFlowsResource",
    "VPCFlowsResourceWithRawResponse",
    "AsyncVPCFlowsResourceWithRawResponse",
    "VPCFlowsResourceWithStreamingResponse",
    "AsyncVPCFlowsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .services import (
    ServicesResource,
    AsyncServicesResource,
    ServicesResourceWithRawResponse,
    AsyncServicesResourceWithRawResponse,
    ServicesResourceWithStreamingResponse,
    AsyncServicesResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["DirectoryResource", "AsyncDirectoryResource"]


class DirectoryResource(SyncAPIResource):
    @cached_property
    def services(self) -> ServicesResource:
        return ServicesResource(self._client)

    @cached_property
    def with_raw_response(self) -> DirectoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DirectoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DirectoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DirectoryResourceWithStreamingResponse(self)


class AsyncDirectoryResource(AsyncAPIResource):
    @cached_property
    def services(self) -> AsyncServicesResource:
        return AsyncServicesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDirectoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDirectoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDirectoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDirectoryResourceWithStreamingResponse(self)


class DirectoryResourceWithRawResponse:
    def __init__(self, directory: DirectoryResource) -> None:
        self._directory = directory

    @cached_property
    def services(self) -> ServicesResourceWithRawResponse:
        return ServicesResourceWithRawResponse(self._directory.services)


class AsyncDirectoryResourceWithRawResponse:
    def __init__(self, directory: AsyncDirectoryResource) -> None:
        self._directory = directory

    @cached_property
    def services(self) -> AsyncServicesResourceWithRawResponse:
        return AsyncServicesResourceWithRawResponse(self._directory.services)


class DirectoryResourceWithStreamingResponse:
    def __init__(self, directory: DirectoryResource) -> None:
        self._directory = directory

    @cached_property
    def services(self) -> ServicesResourceWithStreamingResponse:
        return ServicesResourceWithStreamingResponse(self._directory.services)


class AsyncDirectoryResourceWithStreamingResponse:
    def __init__(self, directory: AsyncDirectoryResource) -> None:
        self._directory = directory

    @cached_property
    def services(self) -> AsyncServicesResourceWithStreamingResponse:
        return AsyncServicesResourceWithStreamingResponse(self._directory.services)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RateLimit", "Action", "ActionResponse", "Bypass", "Match", "MatchHeader", "MatchRequest", "MatchResponse"]


class ActionResponse(BaseModel):
    """A custom content type and reponse to return when the threshold is exceeded.

    The custom response configured in this object will override the custom error for the zone. This object is optional.
    Notes: If you omit this object, Cloudflare will use the default HTML error page. If "mode" is "challenge", "managed_challenge", or "js_challenge", Cloudflare will use the zone challenge pages and you should not provide the "response" object.
    """

    body: Optional[str] = None
    """The response body to return.

    The value must conform to the configured content type.
    """

    content_type: Optional[str] = None
    """The content type of the body.

    Must be one of the following: `text/plain`, `text/xml`, or `application/json`.
    """


class Action(BaseModel):
    """
    The action to perform when the threshold of matched traffic within the configured period is exceeded.
    """

    mode: Optional[Literal["simulate", "ban", "challenge", "js_challenge", "managed_challenge"]] = None
    """The action to perform."""

    response: Optional[ActionResponse] = None
    """A custom content type and reponse to return when the threshold is exceeded.

    The custom response configured in this object will override the custom error for
    the zone. This object is optional. Notes: If you omit this object, Cloudflare
    will use the default HTML error page. If "mode" is "challenge",
    "managed_challenge", or "js_challenge", Cloudflare will use the zone challenge
    pages and you should not provide the "response" object.
    """

    timeout: Optional[float] = None
    """The time in seconds during which Cloudflare will perform the mitigation action.

    Must be an integer value greater than or equal to the period. Notes: If "mode"
    is "challenge", "managed_challenge", or "js_challenge", Cloudflare will use the
    zone's Challenge Passage time and you should not provide this value.
    """


class Bypass(BaseModel):
    name: Optional[Literal["url"]] = None

    value: Optional[str] = None
    """The URL to bypass."""


class MatchHeader(BaseModel):
    name: Optional[str] = None
    """The name of the response header to match."""

    op: Optional[Literal["eq", "ne"]] = None
    """The operator used when matching: `eq` means "equal" and `ne` means "not equal"."""

    value: Optional[str] = None
    """The value of the response header, which must match exactly."""


class MatchRequest(BaseModel):
    methods: Optional[List[Literal["GET", "POST", "PUT", "DELETE", "PATCH", "HEAD", "_ALL_"]]] = None
    """The HTTP methods to match.

    You can specify a subset (for example, `['POST','PUT']`) or all methods
    (`['_ALL_']`). This field is optional when creating a rate limit.
    """

    schemes: Optional[List[str]] = None
    """The HTTP schemes to match.

    You can specify one scheme (`['HTTPS']`), both schemes (`['HTTP','HTTPS']`), or
    all schemes (`['_ALL_']`). This field is optional.
    """

    url: Optional[str] = None
    """
    The URL pattern to match, composed of a host and a path such as
    `example.org/path*`. Normalization is applied before the pattern is matched. `*`
    wildcards are expanded to match applicable traffic. Query strings are not
    matched. Set the value to `*` to match all traffic to your zone.
    """


class MatchResponse(BaseModel):
    origin_traffic: Optional[bool] = None
    """
    When true, only the uncached traffic served from your origin servers will count
    towards rate limiting. In this case, any cached traffic served by Cloudflare
    will not count towards rate limiting. This field is optional. Notes: This field
    is deprecated. Instead, use response headers and set "origin_traffic" to "false"
    to avoid legacy behaviour interacting with the "response_headers" property.
    """


class Match(BaseModel):
    """Determines which traffic the rate limit counts towards the threshold."""

    headers: Optional[List[MatchHeader]] = None

    request: Optional[MatchRequest] = None

    response: Optional[MatchResponse] = None


class RateLimit(BaseModel):
    id: Optional[str] = None
    """The unique identifier of the rate limit."""

    action: Optional[Action] = None
    """
    The action to perform when the threshold of matched traffic within the
    configured period is exceeded.
    """

    bypass: Optional[List[Bypass]] = None
    """Criteria specifying when the current rate limit should be bypassed.

    You can specify that the rate limit should not apply to one or more URLs.
    """

    description: Optional[str] = None
    """An informative summary of the rule.

    This value is sanitized and any tags will be removed.
    """

    disabled: Optional[bool] = None
    """When true, indicates that the rate limit is currently disabled."""

    match: Optional[Match] = None
    """Determines which traffic the rate limit counts towards the threshold."""

    period: Optional[float] = None
    """The time in seconds (an integer value) to count matching traffic.

    If the count exceeds the configured threshold within this period, Cloudflare
    will perform the configured action.
    """

    threshold: Optional[float] = None
    """The threshold that will trigger the configured mitigation action.

    Configure this value along with the `period` property to establish a threshold
    per period.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ScheduleParam"]


class ScheduleParam(TypedDict, total=False):
    """Defines the schedule for activating DNS policies.

    Settable only for `dns` and `dns_resolver` rules.
    """

    fri: str
    """
    Specify the time intervals when the rule is active on Fridays, in the increasing
    order from 00:00-24:00. If this parameter omitted, the rule is deactivated on
    Fridays. API returns a formatted version of this string, which may cause
    Terraform drift if a unformatted value is used.
    """

    mon: str
    """
    Specify the time intervals when the rule is active on Mondays, in the increasing
    order from 00:00-24:00(capped at maximum of 6 time splits). If this parameter
    omitted, the rule is deactivated on Mondays. API returns a formatted version of
    this string, which may cause Terraform drift if a unformatted value is used.
    """

    sat: str
    """
    Specify the time intervals when the rule is active on Saturdays, in the
    increasing order from 00:00-24:00. If this parameter omitted, the rule is
    deactivated on Saturdays. API returns a formatted version of this string, which
    may cause Terraform drift if a unformatted value is used.
    """

    sun: str
    """
    Specify the time intervals when the rule is active on Sundays, in the increasing
    order from 00:00-24:00. If this parameter omitted, the rule is deactivated on
    Sundays. API returns a formatted version of this string, which may cause
    Terraform drift if a unformatted value is used.
    """

    thu: str
    """
    Specify the time intervals when the rule is active on Thursdays, in the
    increasing order from 00:00-24:00. If this parameter omitted, the rule is
    deactivated on Thursdays. API returns a formatted version of this string, which
    may cause Terraform drift if a unformatted value is used.
    """

    time_zone: str
    """Specify the time zone for rule evaluation.

    When a
    [valid time zone city name](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones#List)
    is provided, Gateway always uses the current time for that time zone. When this
    parameter is omitted, Gateway uses the time zone determined from the user's IP
    address. Colo time zone is used when the user's IP address does not resolve to a
    location.
    """

    tue: str
    """
    Specify the time intervals when the rule is active on Tuesdays, in the
    increasing order from 00:00-24:00. If this parameter omitted, the rule is
    deactivated on Tuesdays. API returns a formatted version of this string, which
    may cause Terraform drift if a unformatted value is used.
    """

    wed: str
    """
    Specify the time intervals when the rule is active on Wednesdays, in the
    increasing order from 00:00-24:00. If this parameter omitted, the rule is
    deactivated on Wednesdays. API returns a formatted version of this string, which
    may cause Terraform drift if a unformatted value is used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.realtime_kit import (
    webhook_edit_webhook_params,
    webhook_create_webhook_params,
    webhook_replace_webhook_params,
)
from ...types.realtime_kit.webhook_edit_webhook_response import WebhookEditWebhookResponse
from ...types.realtime_kit.webhook_get_webhooks_response import WebhookGetWebhooksResponse
from ...types.realtime_kit.webhook_create_webhook_response import WebhookCreateWebhookResponse
from ...types.realtime_kit.webhook_delete_webhook_response import WebhookDeleteWebhookResponse
from ...types.realtime_kit.webhook_replace_webhook_response import WebhookReplaceWebhookResponse
from ...types.realtime_kit.webhook_get_webhook_by_id_response import WebhookGetWebhookByIDResponse

__all__ = ["WebhooksResource", "AsyncWebhooksResource"]


class WebhooksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WebhooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return WebhooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebhooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return WebhooksResourceWithStreamingResponse(self)

    def create_webhook(
        self,
        app_id: str,
        *,
        account_id: str,
        events: List[
            Literal[
                "meeting.started",
                "meeting.ended",
                "meeting.participantJoined",
                "meeting.participantLeft",
                "meeting.chatSynced",
                "recording.statusUpdate",
                "livestreaming.statusUpdate",
                "meeting.transcript",
                "meeting.summary",
            ]
        ],
        name: str,
        url: str,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookCreateWebhookResponse:
        """
        Adds a new webhook to an App.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          events: Events that this webhook will get triggered by

          name: Name of the webhook

          url: URL this webhook will send events to

          enabled: Set whether or not the webhook should be active when created

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks", account_id=account_id, app_id=app_id
            ),
            body=maybe_transform(
                {
                    "events": events,
                    "name": name,
                    "url": url,
                    "enabled": enabled,
                },
                webhook_create_webhook_params.WebhookCreateWebhookParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookCreateWebhookResponse,
        )

    def delete_webhook(
        self,
        webhook_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookDeleteWebhookResponse:
        """
        Removes a webhook for the given webhook ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks/{webhook_id}",
                account_id=account_id,
                app_id=app_id,
                webhook_id=webhook_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookDeleteWebhookResponse,
        )

    def edit_webhook(
        self,
        webhook_id: str,
        *,
        account_id: str,
        app_id: str,
        enabled: bool | Omit = omit,
        events: List[
            Literal[
                "meeting.started",
                "meeting.ended",
                "meeting.participantJoined",
                "meeting.participantLeft",
                "recording.statusUpdate",
                "livestreaming.statusUpdate",
                "meeting.chatSynced",
                "meeting.transcript",
                "meeting.summary",
            ]
        ]
        | Omit = omit,
        name: str | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookEditWebhookResponse:
        """
        Edits the webhook details for the given webhook ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          events: Events that the webhook will get triggered by

          name: Name of the webhook

          url: URL the webhook will send events to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks/{webhook_id}",
                account_id=account_id,
                app_id=app_id,
                webhook_id=webhook_id,
            ),
            body=maybe_transform(
                {
                    "enabled": enabled,
                    "events": events,
                    "name": name,
                    "url": url,
                },
                webhook_edit_webhook_params.WebhookEditWebhookParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookEditWebhookResponse,
        )

    def get_webhook_by_id(
        self,
        webhook_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookGetWebhookByIDResponse:
        """
        Returns webhook details for the given webhook ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks/{webhook_id}",
                account_id=account_id,
                app_id=app_id,
                webhook_id=webhook_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookGetWebhookByIDResponse,
        )

    def get_webhooks(
        self,
        app_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookGetWebhooksResponse:
        """
        Returns details of all webhooks for an App.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks", account_id=account_id, app_id=app_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookGetWebhooksResponse,
        )

    def replace_webhook(
        self,
        webhook_id: str,
        *,
        account_id: str,
        app_id: str,
        events: List[
            Literal[
                "meeting.started",
                "meeting.ended",
                "meeting.participantJoined",
                "meeting.participantLeft",
                "meeting.chatSynced",
                "recording.statusUpdate",
                "livestreaming.statusUpdate",
                "meeting.transcript",
                "meeting.summary",
            ]
        ],
        name: str,
        url: str,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookReplaceWebhookResponse:
        """
        Replace all details for the given webhook ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          events: Events that this webhook will get triggered by

          name: Name of the webhook

          url: URL this webhook will send events to

          enabled: Set whether or not the webhook should be active when created

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks/{webhook_id}",
                account_id=account_id,
                app_id=app_id,
                webhook_id=webhook_id,
            ),
            body=maybe_transform(
                {
                    "events": events,
                    "name": name,
                    "url": url,
                    "enabled": enabled,
                },
                webhook_replace_webhook_params.WebhookReplaceWebhookParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookReplaceWebhookResponse,
        )


class AsyncWebhooksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWebhooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWebhooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebhooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncWebhooksResourceWithStreamingResponse(self)

    async def create_webhook(
        self,
        app_id: str,
        *,
        account_id: str,
        events: List[
            Literal[
                "meeting.started",
                "meeting.ended",
                "meeting.participantJoined",
                "meeting.participantLeft",
                "meeting.chatSynced",
                "recording.statusUpdate",
                "livestreaming.statusUpdate",
                "meeting.transcript",
                "meeting.summary",
            ]
        ],
        name: str,
        url: str,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookCreateWebhookResponse:
        """
        Adds a new webhook to an App.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          events: Events that this webhook will get triggered by

          name: Name of the webhook

          url: URL this webhook will send events to

          enabled: Set whether or not the webhook should be active when created

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks", account_id=account_id, app_id=app_id
            ),
            body=await async_maybe_transform(
                {
                    "events": events,
                    "name": name,
                    "url": url,
                    "enabled": enabled,
                },
                webhook_create_webhook_params.WebhookCreateWebhookParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookCreateWebhookResponse,
        )

    async def delete_webhook(
        self,
        webhook_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookDeleteWebhookResponse:
        """
        Removes a webhook for the given webhook ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks/{webhook_id}",
                account_id=account_id,
                app_id=app_id,
                webhook_id=webhook_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookDeleteWebhookResponse,
        )

    async def edit_webhook(
        self,
        webhook_id: str,
        *,
        account_id: str,
        app_id: str,
        enabled: bool | Omit = omit,
        events: List[
            Literal[
                "meeting.started",
                "meeting.ended",
                "meeting.participantJoined",
                "meeting.participantLeft",
                "recording.statusUpdate",
                "livestreaming.statusUpdate",
                "meeting.chatSynced",
                "meeting.transcript",
                "meeting.summary",
            ]
        ]
        | Omit = omit,
        name: str | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookEditWebhookResponse:
        """
        Edits the webhook details for the given webhook ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          events: Events that the webhook will get triggered by

          name: Name of the webhook

          url: URL the webhook will send events to

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks/{webhook_id}",
                account_id=account_id,
                app_id=app_id,
                webhook_id=webhook_id,
            ),
            body=await async_maybe_transform(
                {
                    "enabled": enabled,
                    "events": events,
                    "name": name,
                    "url": url,
                },
                webhook_edit_webhook_params.WebhookEditWebhookParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookEditWebhookResponse,
        )

    async def get_webhook_by_id(
        self,
        webhook_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookGetWebhookByIDResponse:
        """
        Returns webhook details for the given webhook ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks/{webhook_id}",
                account_id=account_id,
                app_id=app_id,
                webhook_id=webhook_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookGetWebhookByIDResponse,
        )

    async def get_webhooks(
        self,
        app_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookGetWebhooksResponse:
        """
        Returns details of all webhooks for an App.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks", account_id=account_id, app_id=app_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookGetWebhooksResponse,
        )

    async def replace_webhook(
        self,
        webhook_id: str,
        *,
        account_id: str,
        app_id: str,
        events: List[
            Literal[
                "meeting.started",
                "meeting.ended",
                "meeting.participantJoined",
                "meeting.participantLeft",
                "meeting.chatSynced",
                "recording.statusUpdate",
                "livestreaming.statusUpdate",
                "meeting.transcript",
                "meeting.summary",
            ]
        ],
        name: str,
        url: str,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookReplaceWebhookResponse:
        """
        Replace all details for the given webhook ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          events: Events that this webhook will get triggered by

          name: Name of the webhook

          url: URL this webhook will send events to

          enabled: Set whether or not the webhook should be active when created

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/webhooks/{webhook_id}",
                account_id=account_id,
                app_id=app_id,
                webhook_id=webhook_id,
            ),
            body=await async_maybe_transform(
                {
                    "events": events,
                    "name": name,
                    "url": url,
                    "enabled": enabled,
                },
                webhook_replace_webhook_params.WebhookReplaceWebhookParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookReplaceWebhookResponse,
        )


class WebhooksResourceWithRawResponse:
    def __init__(self, webhooks: WebhooksResource) -> None:
        self._webhooks = webhooks

        self.create_webhook = to_raw_response_wrapper(
            webhooks.create_webhook,
        )
        self.delete_webhook = to_raw_response_wrapper(
            webhooks.delete_webhook,
        )
        self.edit_webhook = to_raw_response_wrapper(
            webhooks.edit_webhook,
        )
        self.get_webhook_by_id = to_raw_response_wrapper(
            webhooks.get_webhook_by_id,
        )
        self.get_webhooks = to_raw_response_wrapper(
            webhooks.get_webhooks,
        )
        self.replace_webhook = to_raw_response_wrapper(
            webhooks.replace_webhook,
        )


class AsyncWebhooksResourceWithRawResponse:
    def __init__(self, webhooks: AsyncWebhooksResource) -> None:
        self._webhooks = webhooks

        self.create_webhook = async_to_raw_response_wrapper(
            webhooks.create_webhook,
        )
        self.delete_webhook = async_to_raw_response_wrapper(
            webhooks.delete_webhook,
        )
        self.edit_webhook = async_to_raw_response_wrapper(
            webhooks.edit_webhook,
        )
        self.get_webhook_by_id = async_to_raw_response_wrapper(
            webhooks.get_webhook_by_id,
        )
        self.get_webhooks = async_to_raw_response_wrapper(
            webhooks.get_webhooks,
        )
        self.replace_webhook = async_to_raw_response_wrapper(
            webhooks.replace_webhook,
        )


class WebhooksResourceWithStreamingResponse:
    def __init__(self, webhooks: WebhooksResource) -> None:
        self._webhooks = webhooks

        self.create_webhook = to_streamed_response_wrapper(
            webhooks.create_webhook,
        )
        self.delete_webhook = to_streamed_response_wrapper(
            webhooks.delete_webhook,
        )
        self.edit_webhook = to_streamed_response_wrapper(
            webhooks.edit_webhook,
        )
        self.get_webhook_by_id = to_streamed_response_wrapper(
            webhooks.get_webhook_by_id,
        )
        self.get_webhooks = to_streamed_response_wrapper(
            webhooks.get_webhooks,
        )
        self.replace_webhook = to_streamed_response_wrapper(
            webhooks.replace_webhook,
        )


class AsyncWebhooksResourceWithStreamingResponse:
    def __init__(self, webhooks: AsyncWebhooksResource) -> None:
        self._webhooks = webhooks

        self.create_webhook = async_to_streamed_response_wrapper(
            webhooks.create_webhook,
        )
        self.delete_webhook = async_to_streamed_response_wrapper(
            webhooks.delete_webhook,
        )
        self.edit_webhook = async_to_streamed_response_wrapper(
            webhooks.edit_webhook,
        )
        self.get_webhook_by_id = async_to_streamed_response_wrapper(
            webhooks.get_webhook_by_id,
        )
        self.get_webhooks = async_to_streamed_response_wrapper(
            webhooks.get_webhooks,
        )
        self.replace_webhook = async_to_streamed_response_wrapper(
            webhooks.replace_webhook,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .maintenance_config_get_response import MaintenanceConfigGetResponse as MaintenanceConfigGetResponse
from .maintenance_config_update_params import MaintenanceConfigUpdateParams as MaintenanceConfigUpdateParams
from .maintenance_config_update_response import MaintenanceConfigUpdateResponse as MaintenanceConfigUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Redirect"]


class Redirect(BaseModel):
    """The definition of the redirect."""

    source_url: str

    target_url: str

    include_subdomains: Optional[bool] = None

    preserve_path_suffix: Optional[bool] = None

    preserve_query_string: Optional[bool] = None

    status_code: Optional[Literal[301, 302, 307, 308]] = None

    subpath_matching: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ConnectivitySettingEditParams"]


class ConnectivitySettingEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account ID"""

    icmp_proxy_enabled: bool
    """A flag to enable the ICMP proxy for the account network."""

    offramp_warp_enabled: bool
    """A flag to enable WARP to WARP traffic."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.diagnostics import endpoint_healthcheck_create_params, endpoint_healthcheck_update_params
from ...types.diagnostics.endpoint_healthcheck_get_response import EndpointHealthcheckGetResponse
from ...types.diagnostics.endpoint_healthcheck_list_response import EndpointHealthcheckListResponse
from ...types.diagnostics.endpoint_healthcheck_create_response import EndpointHealthcheckCreateResponse
from ...types.diagnostics.endpoint_healthcheck_delete_response import EndpointHealthcheckDeleteResponse
from ...types.diagnostics.endpoint_healthcheck_update_response import EndpointHealthcheckUpdateResponse

__all__ = ["EndpointHealthchecksResource", "AsyncEndpointHealthchecksResource"]


class EndpointHealthchecksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EndpointHealthchecksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EndpointHealthchecksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EndpointHealthchecksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EndpointHealthchecksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        check_type: Literal["icmp"],
        endpoint: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[EndpointHealthcheckCreateResponse]:
        """
        Create Endpoint Health Check.

        Args:
          account_id: Identifier

          check_type: type of check to perform

          endpoint: the IP address of the host to perform checks against

          name: Optional name associated with this check

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/diagnostics/endpoint-healthchecks", account_id=account_id),
            body=maybe_transform(
                {
                    "check_type": check_type,
                    "endpoint": endpoint,
                    "name": name,
                },
                endpoint_healthcheck_create_params.EndpointHealthcheckCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[EndpointHealthcheckCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[EndpointHealthcheckCreateResponse]], ResultWrapper[EndpointHealthcheckCreateResponse]
            ),
        )

    def update(
        self,
        id: str,
        *,
        account_id: str,
        check_type: Literal["icmp"],
        endpoint: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[EndpointHealthcheckUpdateResponse]:
        """
        Update a Endpoint Health Check.

        Args:
          account_id: Identifier

          id: UUID.

          check_type: type of check to perform

          endpoint: the IP address of the host to perform checks against

          name: Optional name associated with this check

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/diagnostics/endpoint-healthchecks/{id}", account_id=account_id, id=id
            ),
            body=maybe_transform(
                {
                    "check_type": check_type,
                    "endpoint": endpoint,
                    "name": name,
                },
                endpoint_healthcheck_update_params.EndpointHealthcheckUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[EndpointHealthcheckUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[EndpointHealthcheckUpdateResponse]], ResultWrapper[EndpointHealthcheckUpdateResponse]
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[EndpointHealthcheckListResponse]:
        """
        List Endpoint Health Checks.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/diagnostics/endpoint-healthchecks", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[EndpointHealthcheckListResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[EndpointHealthcheckListResponse]], ResultWrapper[EndpointHealthcheckListResponse]
            ),
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EndpointHealthcheckDeleteResponse:
        """
        Delete Endpoint Health Check.

        Args:
          account_id: Identifier

          id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/diagnostics/endpoint-healthchecks/{id}", account_id=account_id, id=id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EndpointHealthcheckDeleteResponse,
        )

    def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[EndpointHealthcheckGetResponse]:
        """
        Get a single Endpoint Health Check.

        Args:
          account_id: Identifier

          id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/diagnostics/endpoint-healthchecks/{id}", account_id=account_id, id=id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[EndpointHealthcheckGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[EndpointHealthcheckGetResponse]], ResultWrapper[EndpointHealthcheckGetResponse]),
        )


class AsyncEndpointHealthchecksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEndpointHealthchecksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEndpointHealthchecksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEndpointHealthchecksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEndpointHealthchecksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        check_type: Literal["icmp"],
        endpoint: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[EndpointHealthcheckCreateResponse]:
        """
        Create Endpoint Health Check.

        Args:
          account_id: Identifier

          check_type: type of check to perform

          endpoint: the IP address of the host to perform checks against

          name: Optional name associated with this check

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/diagnostics/endpoint-healthchecks", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "check_type": check_type,
                    "endpoint": endpoint,
                    "name": name,
                },
                endpoint_healthcheck_create_params.EndpointHealthcheckCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[EndpointHealthcheckCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[EndpointHealthcheckCreateResponse]], ResultWrapper[EndpointHealthcheckCreateResponse]
            ),
        )

    async def update(
        self,
        id: str,
        *,
        account_id: str,
        check_type: Literal["icmp"],
        endpoint: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[EndpointHealthcheckUpdateResponse]:
        """
        Update a Endpoint Health Check.

        Args:
          account_id: Identifier

          id: UUID.

          check_type: type of check to perform

          endpoint: the IP address of the host to perform checks against

          name: Optional name associated with this check

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/diagnostics/endpoint-healthchecks/{id}", account_id=account_id, id=id
            ),
            body=await async_maybe_transform(
                {
                    "check_type": check_type,
                    "endpoint": endpoint,
                    "name": name,
                },
                endpoint_healthcheck_update_params.EndpointHealthcheckUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[EndpointHealthcheckUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[EndpointHealthcheckUpdateResponse]], ResultWrapper[EndpointHealthcheckUpdateResponse]
            ),
        )

    async def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[EndpointHealthcheckListResponse]:
        """
        List Endpoint Health Checks.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/diagnostics/endpoint-healthchecks", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[EndpointHealthcheckListResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[EndpointHealthcheckListResponse]], ResultWrapper[EndpointHealthcheckListResponse]
            ),
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EndpointHealthcheckDeleteResponse:
        """
        Delete Endpoint Health Check.

        Args:
          account_id: Identifier

          id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/diagnostics/endpoint-healthchecks/{id}", account_id=account_id, id=id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EndpointHealthcheckDeleteResponse,
        )

    async def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[EndpointHealthcheckGetResponse]:
        """
        Get a single Endpoint Health Check.

        Args:
          account_id: Identifier

          id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/diagnostics/endpoint-healthchecks/{id}", account_id=account_id, id=id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[EndpointHealthcheckGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[EndpointHealthcheckGetResponse]], ResultWrapper[EndpointHealthcheckGetResponse]),
        )


class EndpointHealthchecksResourceWithRawResponse:
    def __init__(self, endpoint_healthchecks: EndpointHealthchecksResource) -> None:
        self._endpoint_healthchecks = endpoint_healthchecks

        self.create = to_raw_response_wrapper(
            endpoint_healthchecks.create,
        )
        self.update = to_raw_response_wrapper(
            endpoint_healthchecks.update,
        )
        self.list = to_raw_response_wrapper(
            endpoint_healthchecks.list,
        )
        self.delete = to_raw_response_wrapper(
            endpoint_healthchecks.delete,
        )
        self.get = to_raw_response_wrapper(
            endpoint_healthchecks.get,
        )


class AsyncEndpointHealthchecksResourceWithRawResponse:
    def __init__(self, endpoint_healthchecks: AsyncEndpointHealthchecksResource) -> None:
        self._endpoint_healthchecks = endpoint_healthchecks

        self.create = async_to_raw_response_wrapper(
            endpoint_healthchecks.create,
        )
        self.update = async_to_raw_response_wrapper(
            endpoint_healthchecks.update,
        )
        self.list = async_to_raw_response_wrapper(
            endpoint_healthchecks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            endpoint_healthchecks.delete,
        )
        self.get = async_to_raw_response_wrapper(
            endpoint_healthchecks.get,
        )


class EndpointHealthchecksResourceWithStreamingResponse:
    def __init__(self, endpoint_healthchecks: EndpointHealthchecksResource) -> None:
        self._endpoint_healthchecks = endpoint_healthchecks

        self.create = to_streamed_response_wrapper(
            endpoint_healthchecks.create,
        )
        self.update = to_streamed_response_wrapper(
            endpoint_healthchecks.update,
        )
        self.list = to_streamed_response_wrapper(
            endpoint_healthchecks.list,
        )
        self.delete = to_streamed_response_wrapper(
            endpoint_healthchecks.delete,
        )
        self.get = to_streamed_response_wrapper(
            endpoint_healthchecks.get,
        )


class AsyncEndpointHealthchecksResourceWithStreamingResponse:
    def __init__(self, endpoint_healthchecks: AsyncEndpointHealthchecksResource) -> None:
        self._endpoint_healthchecks = endpoint_healthchecks

        self.create = async_to_streamed_response_wrapper(
            endpoint_healthchecks.create,
        )
        self.update = async_to_streamed_response_wrapper(
            endpoint_healthchecks.update,
        )
        self.list = async_to_streamed_response_wrapper(
            endpoint_healthchecks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            endpoint_healthchecks.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            endpoint_healthchecks.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.vulnerability_scanner import (
    target_environment_edit_params,
    target_environment_list_params,
    target_environment_create_params,
    target_environment_update_params,
)
from ...types.vulnerability_scanner.target_environment_get_response import TargetEnvironmentGetResponse
from ...types.vulnerability_scanner.target_environment_edit_response import TargetEnvironmentEditResponse
from ...types.vulnerability_scanner.target_environment_list_response import TargetEnvironmentListResponse
from ...types.vulnerability_scanner.target_environment_create_response import TargetEnvironmentCreateResponse
from ...types.vulnerability_scanner.target_environment_update_response import TargetEnvironmentUpdateResponse

__all__ = ["TargetEnvironmentsResource", "AsyncTargetEnvironmentsResource"]


class TargetEnvironmentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TargetEnvironmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TargetEnvironmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TargetEnvironmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TargetEnvironmentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        target: target_environment_create_params.Target,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TargetEnvironmentCreateResponse]:
        """
        Creates a new target environment for the account.

        Args:
          account_id: Identifier.

          name: Human-readable name.

          target: Identifies the Cloudflare asset to scan. Uses a `type` discriminator. Currently
              the service supports only `zone` targets.

          description: Optional description.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/vuln_scanner/target_environments", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "target": target,
                    "description": description,
                },
                target_environment_create_params.TargetEnvironmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TargetEnvironmentCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[TargetEnvironmentCreateResponse]], ResultWrapper[TargetEnvironmentCreateResponse]
            ),
        )

    def update(
        self,
        target_environment_id: str,
        *,
        account_id: str,
        name: str,
        target: target_environment_update_params.Target,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TargetEnvironmentUpdateResponse]:
        """Replaces a target environment.

        All fields must be provided.

        Args:
          account_id: Identifier.

          name: Human-readable name.

          target: Identifies the Cloudflare asset to scan. Uses a `type` discriminator. Currently
              the service supports only `zone` targets.

          description: Optional description.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not target_environment_id:
            raise ValueError(
                f"Expected a non-empty value for `target_environment_id` but received {target_environment_id!r}"
            )
        return self._put(
            path_template(
                "/accounts/{account_id}/vuln_scanner/target_environments/{target_environment_id}",
                account_id=account_id,
                target_environment_id=target_environment_id,
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "target": target,
                    "description": description,
                },
                target_environment_update_params.TargetEnvironmentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TargetEnvironmentUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[TargetEnvironmentUpdateResponse]], ResultWrapper[TargetEnvironmentUpdateResponse]
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[TargetEnvironmentListResponse]:
        """
        Returns all target environments for the account.

        Args:
          account_id: Identifier.

          page: Page number of paginated results.

          per_page: Number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/vuln_scanner/target_environments", account_id=account_id),
            page=SyncV4PagePaginationArray[TargetEnvironmentListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    target_environment_list_params.TargetEnvironmentListParams,
                ),
            ),
            model=TargetEnvironmentListResponse,
        )

    def delete(
        self,
        target_environment_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Removes a target environment.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not target_environment_id:
            raise ValueError(
                f"Expected a non-empty value for `target_environment_id` but received {target_environment_id!r}"
            )
        return self._delete(
            path_template(
                "/accounts/{account_id}/vuln_scanner/target_environments/{target_environment_id}",
                account_id=account_id,
                target_environment_id=target_environment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def edit(
        self,
        target_environment_id: str,
        *,
        account_id: str,
        description: Optional[str] | Omit = omit,
        name: str | Omit = omit,
        target: target_environment_edit_params.Target | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TargetEnvironmentEditResponse]:
        """
        Updates a target environment with only the provided fields; omitted fields
        remain unchanged.

        Args:
          account_id: Identifier.

          description: Optional description. Omit to leave unchanged, set to `null` to clear, or
              provide a string to update.

          name: Human-readable name.

          target: Identifies the Cloudflare asset to scan. Uses a `type` discriminator. Currently
              the service supports only `zone` targets.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not target_environment_id:
            raise ValueError(
                f"Expected a non-empty value for `target_environment_id` but received {target_environment_id!r}"
            )
        return self._patch(
            path_template(
                "/accounts/{account_id}/vuln_scanner/target_environments/{target_environment_id}",
                account_id=account_id,
                target_environment_id=target_environment_id,
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "target": target,
                },
                target_environment_edit_params.TargetEnvironmentEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TargetEnvironmentEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TargetEnvironmentEditResponse]], ResultWrapper[TargetEnvironmentEditResponse]),
        )

    def get(
        self,
        target_environment_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TargetEnvironmentGetResponse]:
        """
        Returns a single target environment by ID.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not target_environment_id:
            raise ValueError(
                f"Expected a non-empty value for `target_environment_id` but received {target_environment_id!r}"
            )
        return self._get(
            path_template(
                "/accounts/{account_id}/vuln_scanner/target_environments/{target_environment_id}",
                account_id=account_id,
                target_environment_id=target_environment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TargetEnvironmentGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TargetEnvironmentGetResponse]], ResultWrapper[TargetEnvironmentGetResponse]),
        )


class AsyncTargetEnvironmentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTargetEnvironmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTargetEnvironmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTargetEnvironmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTargetEnvironmentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        target: target_environment_create_params.Target,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TargetEnvironmentCreateResponse]:
        """
        Creates a new target environment for the account.

        Args:
          account_id: Identifier.

          name: Human-readable name.

          target: Identifies the Cloudflare asset to scan. Uses a `type` discriminator. Currently
              the service supports only `zone` targets.

          description: Optional description.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/vuln_scanner/target_environments", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "target": target,
                    "description": description,
                },
                target_environment_create_params.TargetEnvironmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TargetEnvironmentCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[TargetEnvironmentCreateResponse]], ResultWrapper[TargetEnvironmentCreateResponse]
            ),
        )

    async def update(
        self,
        target_environment_id: str,
        *,
        account_id: str,
        name: str,
        target: target_environment_update_params.Target,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TargetEnvironmentUpdateResponse]:
        """Replaces a target environment.

        All fields must be provided.

        Args:
          account_id: Identifier.

          name: Human-readable name.

          target: Identifies the Cloudflare asset to scan. Uses a `type` discriminator. Currently
              the service supports only `zone` targets.

          description: Optional description.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not target_environment_id:
            raise ValueError(
                f"Expected a non-empty value for `target_environment_id` but received {target_environment_id!r}"
            )
        return await self._put(
            path_template(
                "/accounts/{account_id}/vuln_scanner/target_environments/{target_environment_id}",
                account_id=account_id,
                target_environment_id=target_environment_id,
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "target": target,
                    "description": description,
                },
                target_environment_update_params.TargetEnvironmentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TargetEnvironmentUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[TargetEnvironmentUpdateResponse]], ResultWrapper[TargetEnvironmentUpdateResponse]
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TargetEnvironmentListResponse, AsyncV4PagePaginationArray[TargetEnvironmentListResponse]]:
        """
        Returns all target environments for the account.

        Args:
          account_id: Identifier.

          page: Page number of paginated results.

          per_page: Number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/vuln_scanner/target_environments", account_id=account_id),
            page=AsyncV4PagePaginationArray[TargetEnvironmentListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    target_environment_list_params.TargetEnvironmentListParams,
                ),
            ),
            model=TargetEnvironmentListResponse,
        )

    async def delete(
        self,
        target_environment_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Removes a target environment.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not target_environment_id:
            raise ValueError(
                f"Expected a non-empty value for `target_environment_id` but received {target_environment_id!r}"
            )
        return await self._delete(
            path_template(
                "/accounts/{account_id}/vuln_scanner/target_environments/{target_environment_id}",
                account_id=account_id,
                target_environment_id=target_environment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def edit(
        self,
        target_environment_id: str,
        *,
        account_id: str,
        description: Optional[str] | Omit = omit,
        name: str | Omit = omit,
        target: target_environment_edit_params.Target | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TargetEnvironmentEditResponse]:
        """
        Updates a target environment with only the provided fields; omitted fields
        remain unchanged.

        Args:
          account_id: Identifier.

          description: Optional description. Omit to leave unchanged, set to `null` to clear, or
              provide a string to update.

          name: Human-readable name.

          target: Identifies the Cloudflare asset to scan. Uses a `type` discriminator. Currently
              the service supports only `zone` targets.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not target_environment_id:
            raise ValueError(
                f"Expected a non-empty value for `target_environment_id` but received {target_environment_id!r}"
            )
        return await self._patch(
            path_template(
                "/accounts/{account_id}/vuln_scanner/target_environments/{target_environment_id}",
                account_id=account_id,
                target_environment_id=target_environment_id,
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "target": target,
                },
                target_environment_edit_params.TargetEnvironmentEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TargetEnvironmentEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TargetEnvironmentEditResponse]], ResultWrapper[TargetEnvironmentEditResponse]),
        )

    async def get(
        self,
        target_environment_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TargetEnvironmentGetResponse]:
        """
        Returns a single target environment by ID.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not target_environment_id:
            raise ValueError(
                f"Expected a non-empty value for `target_environment_id` but received {target_environment_id!r}"
            )
        return await self._get(
            path_template(
                "/accounts/{account_id}/vuln_scanner/target_environments/{target_environment_id}",
                account_id=account_id,
                target_environment_id=target_environment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TargetEnvironmentGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TargetEnvironmentGetResponse]], ResultWrapper[TargetEnvironmentGetResponse]),
        )


class TargetEnvironmentsResourceWithRawResponse:
    def __init__(self, target_environments: TargetEnvironmentsResource) -> None:
        self._target_environments = target_environments

        self.create = to_raw_response_wrapper(
            target_environments.create,
        )
        self.update = to_raw_response_wrapper(
            target_environments.update,
        )
        self.list = to_raw_response_wrapper(
            target_environments.list,
        )
        self.delete = to_raw_response_wrapper(
            target_environments.delete,
        )
        self.edit = to_raw_response_wrapper(
            target_environments.edit,
        )
        self.get = to_raw_response_wrapper(
            target_environments.get,
        )


class AsyncTargetEnvironmentsResourceWithRawResponse:
    def __init__(self, target_environments: AsyncTargetEnvironmentsResource) -> None:
        self._target_environments = target_environments

        self.create = async_to_raw_response_wrapper(
            target_environments.create,
        )
        self.update = async_to_raw_response_wrapper(
            target_environments.update,
        )
        self.list = async_to_raw_response_wrapper(
            target_environments.list,
        )
        self.delete = async_to_raw_response_wrapper(
            target_environments.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            target_environments.edit,
        )
        self.get = async_to_raw_response_wrapper(
            target_environments.get,
        )


class TargetEnvironmentsResourceWithStreamingResponse:
    def __init__(self, target_environments: TargetEnvironmentsResource) -> None:
        self._target_environments = target_environments

        self.create = to_streamed_response_wrapper(
            target_environments.create,
        )
        self.update = to_streamed_response_wrapper(
            target_environments.update,
        )
        self.list = to_streamed_response_wrapper(
            target_environments.list,
        )
        self.delete = to_streamed_response_wrapper(
            target_environments.delete,
        )
        self.edit = to_streamed_response_wrapper(
            target_environments.edit,
        )
        self.get = to_streamed_response_wrapper(
            target_environments.get,
        )


class AsyncTargetEnvironmentsResourceWithStreamingResponse:
    def __init__(self, target_environments: AsyncTargetEnvironmentsResource) -> None:
        self._target_environments = target_environments

        self.create = async_to_streamed_response_wrapper(
            target_environments.create,
        )
        self.update = async_to_streamed_response_wrapper(
            target_environments.update,
        )
        self.list = async_to_streamed_response_wrapper(
            target_environments.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            target_environments.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            target_environments.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            target_environments.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .provider import Provider
from ...._models import BaseModel

__all__ = ["Sippy", "Destination", "Source"]


class Destination(BaseModel):
    """Details about the configured destination bucket."""

    access_key_id: Optional[str] = FieldInfo(alias="accessKeyId", default=None)
    """ID of the Cloudflare API token used when writing objects to this bucket."""

    account: Optional[str] = None

    bucket: Optional[str] = None
    """Name of the bucket on the provider."""

    provider: Optional[Provider] = None


class Source(BaseModel):
    """Details about the configured source bucket."""

    bucket: Optional[str] = None
    """Name of the bucket on the provider (AWS, GCS only)."""

    bucket_url: Optional[str] = FieldInfo(alias="bucketUrl", default=None)
    """S3-compatible URL (Generic S3-compatible providers only)."""

    provider: Optional[Literal["aws", "gcs", "s3"]] = None

    region: Optional[str] = None
    """Region where the bucket resides (AWS only)."""


class Sippy(BaseModel):
    destination: Optional[Destination] = None
    """Details about the configured destination bucket."""

    enabled: Optional[bool] = None
    """State of Sippy for this bucket."""

    source: Optional[Source] = None
    """Details about the configured source bucket."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr
from .override_url import OverrideURL
from .waf_rule_param import WAFRuleParam
from .rewrite_action_param import RewriteActionParam

__all__ = ["OverrideUpdateParams"]


class OverrideUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier."""

    id: Required[str]
    """Defines an identifier."""

    rewrite_action: Required[RewriteActionParam]
    """
    Specifies that, when a WAF rule matches, its configured action will be replaced
    by the action configured in this object.
    """

    rules: Required[WAFRuleParam]
    """An object that allows you to override the action of specific WAF rules.

    Each key of this object must be the ID of a WAF rule, and each value must be a
    valid WAF action. Unless you are disabling a rule, ensure that you also enable
    the rule group that this WAF rule belongs to. When creating a new URI-based WAF
    override, you must provide a `groups` object or a `rules` object.
    """

    urls: Required[SequenceNotStr[OverrideURL]]
    """The URLs to include in the current WAF override.

    You can use wildcards. Each entered URL will be escaped before use, which means
    you can only use simple wildcard patterns.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .top import (
    TopResource,
    AsyncTopResource,
    TopResourceWithRawResponse,
    AsyncTopResourceWithRawResponse,
    TopResourceWithStreamingResponse,
    AsyncTopResourceWithStreamingResponse,
)
from .locations import (
    LocationsResource,
    AsyncLocationsResource,
    LocationsResourceWithRawResponse,
    AsyncLocationsResourceWithRawResponse,
    LocationsResourceWithStreamingResponse,
    AsyncLocationsResourceWithStreamingResponse,
)

__all__ = [
    "LocationsResource",
    "AsyncLocationsResource",
    "LocationsResourceWithRawResponse",
    "AsyncLocationsResourceWithRawResponse",
    "LocationsResourceWithStreamingResponse",
    "AsyncLocationsResourceWithStreamingResponse",
    "TopResource",
    "AsyncTopResource",
    "TopResourceWithRawResponse",
    "AsyncTopResourceWithRawResponse",
    "TopResourceWithStreamingResponse",
    "AsyncTopResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["LockdownListParams"]


class LockdownListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier."""

    created_on: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The timestamp of when the rule was created."""

    description: str
    """A string to search for in the description of existing rules."""

    description_search: str
    """A string to search for in the description of existing rules."""

    ip: str
    """A single IP address to search for in existing rules."""

    ip_range_search: str
    """A single IP address range to search for in existing rules."""

    ip_search: str
    """A single IP address to search for in existing rules."""

    modified_on: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The timestamp of when the rule was last modified."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """The maximum number of results per page.

    You can only set the value to `1` or to a multiple of 5 such as `5`, `10`, `15`,
    or `20`.
    """

    priority: float
    """The priority of the rule to control the processing order.

    A lower number indicates higher priority. If not provided, any rules with a
    configured priority will be processed before rules without a priority.
    """

    uri_search: str
    """A single URI to search for in the list of URLs of existing rules."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .kind import Kind
from ..ips.ips import IPs
from ..._models import BaseModel

__all__ = ["AddressMapCreateResponse", "Membership"]


class Membership(BaseModel):
    can_delete: Optional[bool] = None
    """Controls whether the membership can be deleted via the API or not."""

    created_at: Optional[datetime] = None

    identifier: Optional[str] = None
    """The identifier for the membership (eg. a zone or account tag)."""

    kind: Optional[Kind] = None
    """The type of the membership."""


class AddressMapCreateResponse(BaseModel):
    id: Optional[str] = None
    """Identifier of an Address Map."""

    can_delete: Optional[bool] = None
    """If set to false, then the Address Map cannot be deleted via API.

    This is true for Cloudflare-managed maps.
    """

    can_modify_ips: Optional[bool] = None
    """If set to false, then the IPs on the Address Map cannot be modified via the API.

    This is true for Cloudflare-managed maps.
    """

    created_at: Optional[datetime] = None

    default_sni: Optional[str] = None
    """
    If you have legacy TLS clients which do not send the TLS server name indicator,
    then you can specify one default SNI on the map. If Cloudflare receives a TLS
    handshake from a client without an SNI, it will respond with the default SNI on
    those IPs. The default SNI can be any valid zone or subdomain owned by the
    account.
    """

    description: Optional[str] = None
    """
    An optional description field which may be used to describe the types of IPs or
    zones on the map.
    """

    enabled: Optional[bool] = None
    """Whether the Address Map is enabled or not.

    Cloudflare's DNS will not respond with IP addresses on an Address Map until the
    map is enabled.
    """

    ips: Optional[IPs] = None
    """The set of IPs on the Address Map."""

    memberships: Optional[List[Membership]] = None
    """Zones and Accounts which will be assigned IPs on this Address Map.

    A zone membership will take priority over an account membership.
    """

    modified_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["EntryUpdateParams"]


class EntryUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Specify the identifier of the hostname."""

    identifier: Required[str]
    """Specify the identifier of the hostname."""

    content: Required[str]
    """Specify the CID or content path of content to block."""

    type: Required[Literal["cid", "content_path"]]
    """Specify the type of content list entry to block."""

    description: str
    """Specify an optional description of the content list entry."""
